#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/4/2 22:40
Desc: 真气网-空气质量
https://www.zq12369.com/environment.php
空气质量在线监测分析平台的空气质量数据
https://www.aqistudy.cn/
"""

import json
import os
import re
from io import StringIO

import pandas as pd
import requests
from py_mini_racer import MiniRacer

from akshare.utils import demjson


def _get_js_path(name: str = None, module_file: str = None) -> str:
    """
    获取 JS 文件的路径(从模块所在目录查找)
    :param name: 文件名
    :type name: str
    :param module_file: 模块路径
    :type module_file: str
    :return: 路径
    :rtype: str
    """
    module_folder = os.path.abspath(os.path.dirname(os.path.dirname(module_file)))
    module_json_path = os.path.join(module_folder, "air", name)
    return module_json_path


def _get_file_content(file_name: str = "crypto.js") -> str:
    """
    获取 JS 文件的内容
    :param file_name:  JS 文件名
    :type file_name: str
    :return: 文件内容
    :rtype: str
    """
    setting_file_name = file_name
    setting_file_path = _get_js_path(setting_file_name, __file__)
    with open(setting_file_path) as f:
        file_data = f.read()
    return file_data


def has_month_data(href):
    """
    Deal with href node
    :param href: href
    :type href: str
    :return: href result
    :rtype: str
    """
    return href and re.compile("monthdata.php").search(href)


def air_city_table() -> pd.DataFrame:
    """
    真气网-空气质量历史数据查询-全部城市列表
    https://www.zq12369.com/environment.php?date=2019-06-05&tab=rank&order=DESC&type=DAY#rank
    :return: 城市映射
    :rtype: pandas.DataFrame
    """
    url = "https://www.zq12369.com/environment.php"
    date = "2020-05-01"
    temp_df = None
    if len(date.split("-")) == 3:
        params = {
            "date": date,
            "tab": "rank",
            "order": "DESC",
            "type": "DAY",
        }
        r = requests.get(url, params=params)
        temp_df = pd.read_html(StringIO(r.text))[1].iloc[1:, :]
        del temp_df["降序"]
        temp_df.reset_index(inplace=True)
        temp_df["index"] = temp_df.index + 1
        temp_df.columns = [
            "序号",
            "省份",
            "城市",
            "AQI",
            "空气质量",
            "PM2.5浓度",
            "首要污染物",
        ]
        temp_df["AQI"] = pd.to_numeric(temp_df["AQI"])
    return temp_df


def air_quality_watch_point(
    city: str = "杭州", start_date: str = "20220408", end_date: str = "20220409"
) -> pd.DataFrame:
    """
    真气网-监测点空气质量-细化到具体城市的每个监测点
    指定之间段之间的空气质量数据
    https://www.zq12369.com/
    :param city: 调用 ak.air_city_table() 接口获取
    :type city: str
    :param start_date: e.g., "20190327"
    :type start_date: str
    :param end_date: e.g., ""20200327""
    :type end_date: str
    :return: 指定城市指定日期区间的观测点空气质量
    :rtype: pandas.DataFrame
    """
    start_date = "-".join([start_date[:4], start_date[4:6], start_date[6:]])
    end_date = "-".join([end_date[:4], end_date[4:6], end_date[6:]])
    url = "https://www.zq12369.com/api/zhenqiapi.php"
    file_data = _get_file_content(file_name="crypto.js")
    ctx = MiniRacer()
    ctx.eval(file_data)
    method = "GETCITYPOINTAVG"
    city_param = ctx.call("encode_param", city)
    payload = {
        "appId": "a01901d3caba1f362d69474674ce477f",
        "method": ctx.call("encode_param", method),
        "city": city_param,
        "startTime": ctx.call("encode_param", start_date),
        "endTime": ctx.call("encode_param", end_date),
        "secret": ctx.call("encode_secret", method, city_param, start_date, end_date),
    }
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/81.0.4044.122 Safari/537.36"
    }
    r = requests.post(url, data=payload, headers=headers)
    data_text = r.text
    data_json = demjson.decode(ctx.call("decode_result", data_text))
    temp_df = pd.DataFrame(data_json["rows"])
    return temp_df


def air_quality_hist(
    city: str = "杭州",
    period: str = "day",
    start_date: str = "20190327",
    end_date: str = "20200427",
) -> pd.DataFrame:
    """
    真气网-空气历史数据
    https://www.zq12369.com/
    :param city: 调用 ak.air_city_table() 接口获取所有城市列表
    :type city: str
    :param period: "hour": 每小时一个数据, 由于数据量比较大, 下载较慢; "day": 每天一个数据; "month": 每个月一个数据
    :type period: str
    :param start_date: e.g., "20190327"
    :type start_date: str
    :param end_date: e.g., "20200327"
    :type end_date: str
    :return: 指定城市和数据频率下在指定时间段内的空气质量数据
    :rtype: pandas.DataFrame
    """
    start_date = "-".join([start_date[:4], start_date[4:6], start_date[6:]])
    end_date = "-".join([end_date[:4], end_date[4:6], end_date[6:]])
    url = "https://www.zq12369.com/api/newzhenqiapi.php"
    file_data = _get_file_content(file_name="outcrypto.js")
    ctx = MiniRacer()
    ctx.eval(file_data)
    app_id = "4f0e3a273d547ce6b7147bfa7ceb4b6e"
    method = "CETCITYPERIOD"
    timestamp = ctx.eval("timestamp = new Date().getTime()")
    p_text = json.dumps(
        {
            "city": city,
            "endTime": f"{end_date} 23:45:39",
            "startTime": f"{start_date} 00:00:00",
            "type": period.upper(),
        },
        ensure_ascii=False,
        indent=None,
    ).replace(' "', '"')
    secret = ctx.call("hex_md5", app_id + method + str(timestamp) + "WEB" + p_text)
    payload = {
        "appId": "4f0e3a273d547ce6b7147bfa7ceb4b6e",
        "method": "CETCITYPERIOD",
        "timestamp": int(timestamp),
        "clienttype": "WEB",
        "object": {
            "city": city,
            "type": period.upper(),
            "startTime": f"{start_date} 00:00:00",
            "endTime": f"{end_date} 23:45:39",
        },
        "secret": secret,
    }
    need = (
        json.dumps(payload, ensure_ascii=False, indent=None, sort_keys=False)
        .replace(' "', '"')
        .replace("\\", "")
        .replace('p": ', 'p":')
        .replace('t": ', 't":')
    )

    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko)"
        "Chrome/100.0.4896.75 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
    }
    params = {"param": ctx.call("encode_param", need)}
    r = requests.post(url, data=params, headers=headers)
    temp_text = ctx.call("decryptData", r.text)
    data_json = demjson.decode(ctx.call("b.decode", temp_text))
    temp_df = pd.DataFrame(data_json["result"]["data"]["rows"])
    temp_df.index = temp_df["time"]
    del temp_df["time"]
    temp_df = temp_df.astype(float, errors="ignore")
    return temp_df


def air_quality_rank(date: str = "") -> pd.DataFrame:
    """
    真气网-168 城市 AQI 排行榜
    https://www.zq12369.com/environment.php?date=2020-03-12&tab=rank&order=DESC&type=DAY#rank
    :param date: "": 当前时刻空气质量排名; "20200312": 当日空气质量排名; "202003": 当月空气质量排名; "2019": 当年空气质量排名;
    :type date: str
    :return: 指定 date 类型的空气质量排名数据
    :rtype: pandas.DataFrame
    """
    if len(date) == 4:
        date = date
    elif len(date) == 6:
        date = "-".join([date[:4], date[4:6]])
    elif date == "":
        date = "实时"
    else:
        date = "-".join([date[:4], date[4:6], date[6:]])

    url = "https://www.zq12369.com/environment.php"

    if len(date.split("-")) == 3:
        params = {
            "date": date,
            "tab": "rank",
            "order": "DESC",
            "type": "DAY",
        }
        r = requests.get(url, params=params)
        return pd.read_html(StringIO(r.text))[1].iloc[1:, :]
    elif len(date.split("-")) == 2:
        params = {
            "month": date,
            "tab": "rank",
            "order": "DESC",
            "type": "MONTH",
        }
        r = requests.get(url, params=params)
        return pd.read_html(StringIO(r.text))[2].iloc[1:, :]
    elif len(date.split("-")) == 1 and date != "实时":
        params = {
            "year": date,
            "tab": "rank",
            "order": "DESC",
            "type": "YEAR",
        }
        r = requests.get(url, params=params)
        return pd.read_html(StringIO(r.text))[3].iloc[1:, :]
    if date == "实时":
        params = {
            "tab": "rank",
            "order": "DESC",
            "type": "MONTH",
        }
        r = requests.get(url, params=params)
        return pd.read_html(StringIO(r.text))[0].iloc[1:, :]


if __name__ == "__main__":
    air_city_table_df = air_city_table()
    print(air_city_table_df)

    air_quality_watch_point_df = air_quality_watch_point(
        city="杭州", start_date="20220408", end_date="20220409"
    )
    print(air_quality_watch_point_df)

    air_quality_hist_df = air_quality_hist(
        city="北京",
        period="day",
        start_date="20220801",
        end_date="20240402",
    )
    print(air_quality_hist_df)

    air_quality_rank_df = air_quality_rank()
    print(air_quality_rank_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/4/4 18:20
Desc: 增发和配股
东方财富网-数据中心-新股数据-增发-全部增发
https://data.eastmoney.com/other/gkzf.html
东方财富网-数据中心-新股数据-配股
https://data.eastmoney.com/xg/pg/
"""

import pandas as pd
import requests

from akshare.utils.tqdm import get_tqdm


def stock_qbzf_em() -> pd.DataFrame:
    """
    东方财富网-数据中心-新股数据-增发-全部增发
    https://data.eastmoney.com/other/gkzf.html
    :return: 全部增发
    :rtype: pandas.DataFrame
    """
    import warnings

    warnings.filterwarnings(action="ignore", category=FutureWarning)
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "ISSUE_DATE",
        "sortTypes": "-1",
        "pageSize": "500",
        "pageNumber": "1",
        "reportName": "RPT_SEO_DETAIL",
        "columns": "ALL",
        "quoteColumns": "f2~01~SECURITY_CODE~NEW_PRICE",
        "quoteType": "0",
        "source": "WEB",
        "client": "WEB",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    total_page = data_json["result"]["pages"]
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    for page in tqdm(range(1, total_page + 1), leave=False):
        params.update(
            {
                "pageNumber": page,
            }
        )
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)

    big_df.rename(
        columns={
            "SECURITY_NAME_ABBR": "股票简称",
            "SECURITY_CODE": "股票代码",
            "CORRECODE": "增发代码",
            "SEO_TYPE": "发行方式",
            "ISSUE_NUM": "发行总数",
            "ONLINE_ISSUE_NUM": "网上发行",
            "ISSUE_PRICE": "发行价格",
            "NEW_PRICE": "最新价",
            "ISSUE_DATE": "发行日期",
            "ISSUE_LISTING_DATE": "增发上市日期",
            "LOCKIN_PERIOD": "锁定期",
        },
        inplace=True,
    )
    big_df = big_df[
        [
            "股票代码",
            "股票简称",
            "增发代码",
            "发行方式",
            "发行总数",
            "网上发行",
            "发行价格",
            "最新价",
            "发行日期",
            "增发上市日期",
            "锁定期",
        ]
    ]
    big_df["发行总数"] = pd.to_numeric(big_df["发行总数"], errors="coerce")
    big_df["发行价格"] = pd.to_numeric(big_df["发行价格"], errors="coerce")
    big_df["最新价"] = pd.to_numeric(big_df["最新价"], errors="coerce")
    big_df["发行方式"] = big_df["发行方式"].map({"2": "公开增发", "1": "定向增发"})
    big_df["发行日期"] = pd.to_datetime(big_df["发行日期"], errors="coerce").dt.date
    big_df["增发上市日期"] = pd.to_datetime(
        big_df["增发上市日期"], errors="coerce"
    ).dt.date
    return big_df


def stock_pg_em() -> pd.DataFrame:
    """
    东方财富网-数据中心-新股数据-配股
    https://data.eastmoney.com/xg/pg/
    :return: 配股
    :rtype: pandas.DataFrame
    """
    import warnings

    warnings.filterwarnings(action="ignore", category=FutureWarning)
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "EQUITY_RECORD_DATE",
        "sortTypes": "-1",
        "pageSize": "50000",
        "pageNumber": "1",
        "reportName": "RPT_IPO_ALLOTMENT",
        "columns": "ALL",
        "quoteColumns": "f2~01~SECURITY_CODE~NEW_PRICE",
        "quoteType": "0",
        "source": "WEB",
        "client": "WEB",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    total_page = data_json["result"]["pages"]
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    for page in tqdm(range(1, total_page + 1), leave=False):
        params.update(
            {
                "pageNumber": page,
            }
        )
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
    big_df.columns = [
        "_",
        "_",
        "股票代码",
        "-",
        "股票简称",
        "配售代码",
        "_",
        "配股比例",
        "配股价",
        "配股前总股本",
        "配股数量",
        "配股后总股本",
        "股权登记日",
        "缴款起始日期",
        "缴款截止日期",
        "上市日",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "最新价",
    ]
    big_df = big_df[
        [
            "股票代码",
            "股票简称",
            "配售代码",
            "配股数量",
            "配股比例",
            "配股价",
            "最新价",
            "配股前总股本",
            "配股后总股本",
            "股权登记日",
            "缴款起始日期",
            "缴款截止日期",
            "上市日",
        ]
    ]
    big_df["配股比例"] = "10配" + big_df["配股比例"].astype(str)
    big_df["配股数量"] = pd.to_numeric(big_df["配股数量"], errors="coerce")
    big_df["配股价"] = pd.to_numeric(big_df["配股价"], errors="coerce")
    big_df["最新价"] = pd.to_numeric(big_df["最新价"], errors="coerce")
    big_df["配股前总股本"] = pd.to_numeric(big_df["配股前总股本"], errors="coerce")
    big_df["配股后总股本"] = pd.to_numeric(big_df["配股后总股本"], errors="coerce")
    big_df["股权登记日"] = pd.to_datetime(big_df["股权登记日"], errors="coerce").dt.date
    big_df["缴款起始日期"] = pd.to_datetime(
        big_df["缴款起始日期"], errors="coerce"
    ).dt.date
    big_df["缴款截止日期"] = pd.to_datetime(
        big_df["缴款截止日期"], errors="coerce"
    ).dt.date
    big_df["上市日"] = pd.to_datetime(big_df["上市日"], errors="coerce").dt.date
    return big_df


if __name__ == "__main__":
    stock_qbzf_em_df = stock_qbzf_em()
    print(stock_qbzf_em_df)

    stock_pg_em_df = stock_pg_em()
    print(stock_pg_em_df)



# !/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/5/16 19:00
Desc: 雪球-个股-公司概况-公司简介
https://xueqiu.com/snowman/S/SH601127/detail#/GSJJ
"""

import pandas as pd
import requests

from akshare.utils.cons import headers


def stock_individual_basic_info_xq(
    symbol: str = "SH601127", token: str = None, timeout: float = None
) -> pd.DataFrame:
    """
    雪球-个股-公司概况-公司简介
    https://xueqiu.com/snowman/S/SH601127/detail#/GSJJ
    :param symbol: 证券代码
    :type symbol: str
    :param token: 雪球财经的 token
    :type token: str
    :param timeout: 设置超时时间
    :type timeout: float
    :return: 公司简介
    :rtype: pandas.DataFrame
    """
    from akshare.stock.cons import xq_a_token
    xq_a_token = token or xq_a_token
    url = "https://stock.xueqiu.com/v5/stock/f10/cn/company.json"
    params = {
        "symbol": symbol,
    }
    headers.update({"cookie": f"xq_a_token={xq_a_token};"})
    r = requests.get(url, params=params, headers=headers, timeout=timeout)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"])
    temp_df.reset_index(inplace=True)
    temp_df.columns = ["item", "value"]
    return temp_df


def stock_individual_basic_info_us_xq(
    symbol: str = "NVDA", token: str = None, timeout: float = None
) -> pd.DataFrame:
    """
    雪球-个股-公司概况-公司简介
    https://xueqiu.com/snowman/S/NVDA/detail#/GSJJ
    :param symbol: 证券代码
    :type symbol: str
    :param token: 雪球财经的 token
    :type token: str
    :param timeout: 设置超时时间
    :type timeout: float
    :return: 公司简介
    :rtype: pandas.DataFrame
    """
    from akshare.stock.cons import xq_a_token
    xq_a_token = token or xq_a_token
    url = "https://stock.xueqiu.com/v5/stock/f10/us/company.json"
    params = {
        "symbol": symbol,
    }
    headers.update({"cookie": f"xq_a_token={xq_a_token};"})
    r = requests.get(url, params=params, headers=headers, timeout=timeout)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"])
    temp_df.reset_index(inplace=True)
    temp_df.columns = ["item", "value"]
    return temp_df


def stock_individual_basic_info_hk_xq(
    symbol: str = "02097", token: str = None, timeout: float = None
) -> pd.DataFrame:
    """
    雪球-个股-公司概况-公司简介
    https://xueqiu.com/S/00700
    :param symbol: 证券代码
    :type symbol: str
    :param token: 雪球财经的 token
    :type token: str
    :param timeout: 设置超时时间
    :type timeout: float
    :return: 公司简介
    :rtype: pandas.DataFrame
    """
    from akshare.stock.cons import xq_a_token
    xq_a_token = token or xq_a_token
    url = "https://stock.xueqiu.com/v5/stock/f10/hk/company.json"
    params = {
        "symbol": symbol,
    }
    headers.update({"cookie": f"xq_a_token={xq_a_token};"})
    r = requests.get(url, params=params, headers=headers, timeout=timeout)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"])
    temp_df.reset_index(inplace=True)
    temp_df.columns = ["item", "value"]
    return temp_df


if __name__ == "__main__":
    stock_individual_basic_info_xq_df = stock_individual_basic_info_xq(
        symbol="SH601127"
    )
    print(stock_individual_basic_info_xq_df)

    stock_us_individual_basic_info_us_xq_df = stock_individual_basic_info_us_xq(
        symbol="NVDA"
    )
    print(stock_us_individual_basic_info_us_xq_df)

    stock_individual_basic_info_hk_xq_df = stock_individual_basic_info_hk_xq(
        symbol="02097"
    )
    print(stock_individual_basic_info_hk_xq_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/9/20 18:00
Desc: 99 期货网-大宗商品库存数据
https://www.99qh.com/
"""

import json
from datetime import datetime
from functools import lru_cache

import pandas as pd
import requests
from bs4 import BeautifulSoup


@lru_cache(maxsize=32)
def __get_99_symbol_map() -> pd.DataFrame:
    """
    99 期货网-品种代码对照表
    https://www.99qh.com/data/stockIn?productId=12
    :return: 品种代码对照表
    :rtype: pandas.DataFrame
    """
    url = "https://www.99qh.com/data/stockIn"
    r = requests.get(url)
    soup = BeautifulSoup(r.text, features="lxml")
    raw_data = soup.find(attrs={"id": "__NEXT_DATA__"}).text
    data_json = json.loads(raw_data)
    df_list = []
    for i, item in enumerate(
        data_json["props"]["pageProps"]["data"]["varietyListData"]
    ):
        temp_df = pd.DataFrame(
            data_json["props"]["pageProps"]["data"]["varietyListData"][i]["productList"]
        )
        df_list.append(temp_df)

    big_df = pd.concat(df_list, ignore_index=True)
    return big_df


def futures_inventory_99(symbol: str = "豆一") -> pd.DataFrame:
    """
    99 期货网-大宗商品库存数据
    https://www.99qh.com/data/stockIn?productId=12
    :param symbol: 交易所对应的具体品种; 如：大连商品交易所的 豆一
    :type symbol: str
    :return: 大宗商品库存数据
    :rtype: pandas.DataFrame
    """
    temp_df = __get_99_symbol_map()
    symbol_name_map = dict(zip(temp_df["name"], temp_df["productId"]))
    symbol_code_map = dict(zip(temp_df["code"], temp_df["productId"]))
    if symbol in symbol_name_map:  # 如果输入的是中文名称
        product_id = symbol_name_map[symbol]
    elif symbol in symbol_code_map:  # 如果输入的是代码
        product_id = symbol_code_map[symbol]
    else:
        raise ValueError(f"未找到品种 {symbol} 对应的编号")

    url = "https://centerapi.fx168api.com/app/qh/api/stock/trend"
    headers = {
        "Content-Type": "application/json;charset=UTF-8",
        "_pcc": "oCs6gFrFr91rvDedcUMm4ZlFMESB+Ur2ZaPLHUPGM0AcbM8CqJ3JQ7OsYAb/i6rjc9936EDAFHq3Vy"
                "YnrPw0pu8RZH6T487oyof7orI3awLt8Ih31Y5uVNzPO2AKDQcud1a4D0/90hu6ZykAW/f+3lVzqaY0Fv8c8jJV/Vg+x/M=",
        "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
                      "Chrome/58.0.3029.110 Safari/537.3",
        "referer": "https://www.99qh.com",
        "origin": "https://www.99qh.com",
    }
    params = {
        "productId": product_id,
        "type": "1",
        "pageNo": "1",
        "pageSize": "5000",
        "startDate": "",
        "endDate": f"{datetime.now().date().isoformat()}",
        "appCategory": "web",
    }
    r = requests.get(url, params, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"]["list"])
    temp_df.columns = ["日期", "收盘价", "库存"]
    temp_df.sort_values(by=["日期"], ignore_index=True, inplace=True)
    temp_df["日期"] = pd.to_datetime(temp_df["日期"], errors="coerce").dt.date
    temp_df["收盘价"] = pd.to_numeric(temp_df["收盘价"], errors="coerce")
    temp_df["库存"] = pd.to_numeric(temp_df["库存"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    futures_inventory_99_df = futures_inventory_99(symbol="豆一")
    print(futures_inventory_99_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2023/7/8 17:15
Desc: 东方财富个股人气榜
https://guba.eastmoney.com/rank/
"""

import pandas as pd
import requests


def stock_hot_rank_em() -> pd.DataFrame:
    """
    东方财富-个股人气榜-人气榜
    https://guba.eastmoney.com/rank/
    :return: 人气榜
    :rtype: pandas.DataFrame
    """
    url = "https://emappdata.eastmoney.com/stockrank/getAllCurrentList"
    payload = {
        "appId": "appId01",
        "globalId": "786e4c21-70dc-435a-93bb-38",
        "marketType": "",
        "pageNo": 1,
        "pageSize": 100,
    }
    r = requests.post(url, json=payload)
    data_json = r.json()
    temp_rank_df = pd.DataFrame(data_json["data"])

    temp_rank_df["mark"] = [
        "0" + "." + item[2:] if "SZ" in item else "1" + "." + item[2:]
        for item in temp_rank_df["sc"]
    ]
    ",".join(temp_rank_df["mark"]) + "?v=08926209912590994"
    params = {
        "ut": "f057cbcbce2a86e2866ab8877db1d059",
        "fltt": "2",
        "invt": "2",
        "fields": "f14,f3,f12,f2",
        "secids": ",".join(temp_rank_df["mark"]) + ",?v=08926209912590994",
    }
    url = "https://push2.eastmoney.com/api/qt/ulist.np/get"
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"]["diff"])
    temp_df.columns = ["最新价", "涨跌幅", "代码", "股票名称"]
    temp_df["最新价"] = pd.to_numeric(temp_df["最新价"], errors="coerce")
    temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"], errors="coerce")
    temp_df["涨跌额"] = temp_df["最新价"] * temp_df["涨跌幅"] / 100
    temp_df["当前排名"] = temp_rank_df["rk"]
    temp_df["代码"] = temp_rank_df["sc"]
    temp_df = temp_df[
        [
            "当前排名",
            "代码",
            "股票名称",
            "最新价",
            "涨跌额",
            "涨跌幅",
        ]
    ]
    temp_df["当前排名"] = pd.to_numeric(temp_df["当前排名"], errors="coerce")
    return temp_df


def stock_hot_rank_detail_em(symbol: str = "SZ000665") -> pd.DataFrame:
    """
    东方财富-个股人气榜-历史趋势及粉丝特征
    https://guba.eastmoney.com/rank/stock?code=000665
    :param symbol: 带市场表示的证券代码
    :type symbol: str
    :return: 个股的历史趋势及粉丝特征
    :rtype: pandas.DataFrame
    """
    url_rank = "https://emappdata.eastmoney.com/stockrank/getHisList"
    payload = {
        "appId": "appId01",
        "globalId": "786e4c21-70dc-435a-93bb-38",
        "marketType": "",
        "srcSecurityCode": symbol,
        "yearType": "5",
    }
    r = requests.post(url_rank, json=payload)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"])
    temp_df["证券代码"] = symbol
    temp_df.columns = ["时间", "排名", "证券代码"]
    temp_df = temp_df[["时间", "排名", "证券代码"]]

    url_follow = "https://emappdata.eastmoney.com/stockrank/getHisProfileList"
    r = requests.post(url_follow, json=payload)
    data_json = r.json()
    temp_df["新晋粉丝"] = (
        pd.DataFrame(data_json["data"])["newUidRate"].str.strip("%").astype(float) / 100
    )
    temp_df["铁杆粉丝"] = (
        pd.DataFrame(data_json["data"])["oldUidRate"].str.strip("%").astype(float) / 100
    )
    temp_df.sort_values(['时间'], inplace=True)
    return temp_df


def stock_hot_rank_detail_realtime_em(symbol: str = "SZ000665") -> pd.DataFrame:
    """
    东方财富-个股人气榜-实时变动
    https://guba.eastmoney.com/rank/stock?code=000665
    :param symbol: 带市场表示的证券代码
    :type symbol: str
    :return: 实时变动
    :rtype: pandas.DataFrame
    """
    url = "https://emappdata.eastmoney.com/stockrank/getCurrentList"
    payload = {
        "appId": "appId01",
        "globalId": "786e4c21-70dc-435a-93bb-38",
        "marketType": "",
        "srcSecurityCode": symbol,
    }
    r = requests.post(url, json=payload)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"])
    temp_df.columns = ["时间", "排名"]
    return temp_df


def stock_hot_keyword_em(symbol: str = "SZ000665") -> pd.DataFrame:
    """
    东方财富-个股人气榜-热门关键词
    https://guba.eastmoney.com/rank/stock?code=000665
    :param symbol: 带市场表示的证券代码
    :type symbol: str
    :return: 热门关键词
    :rtype: pandas.DataFrame
    """
    url = "https://emappdata.eastmoney.com/stockrank/getHotStockRankList"
    payload = {
        "appId": "appId01",
        "globalId": "786e4c21-70dc-435a-93bb-38",
        "srcSecurityCode": symbol,
    }
    r = requests.post(url, json=payload)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"])
    del temp_df["flag"]
    temp_df.columns = ["时间", "股票代码", "概念名称", "概念代码", "热度"]
    return temp_df


def stock_hot_rank_latest_em(symbol: str = "SZ000665") -> pd.DataFrame:
    """
    东方财富-个股人气榜-最新排名
    https://guba.eastmoney.com/rank/stock?code=000665
    :param symbol: 带市场表示的证券代码
    :type symbol: str
    :return: 最新排名
    :rtype: pandas.DataFrame
    """
    url = "https://emappdata.eastmoney.com/stockrank/getCurrentLatest"
    payload = {
        "appId": "appId01",
        "globalId": "786e4c21-70dc-435a-93bb-38",
        "marketType": "",
        "srcSecurityCode": symbol,
    }
    r = requests.post(url, json=payload)
    data_json = r.json()
    temp_df = pd.DataFrame.from_dict(data_json["data"], orient="index")
    temp_df.reset_index(inplace=True)
    temp_df.columns = ["item", "value"]
    return temp_df


def stock_hot_rank_relate_em(symbol: str = "SZ000665") -> pd.DataFrame:
    """
    东方财富-个股人气榜-相关股票
    https://guba.eastmoney.com/rank/stock?code=000665
    :param symbol: 带市场表示的证券代码
    :type symbol: str
    :return: 相关股票
    :rtype: pandas.DataFrame
    """
    url = "https://emappdata.eastmoney.com/stockrank/getFollowStockRankList"
    payload = {
        "appId": "appId01",
        "globalId": "786e4c21-70dc-435a-93bb-38",
        "srcSecurityCode": symbol,
    }
    r = requests.post(url, json=payload)
    data_json = r.json()
    temp_df = pd.DataFrame.from_dict(data_json["data"])
    temp_df.columns = ["时间", "-", "股票代码", "-", "相关股票代码", "涨跌幅", "-"]
    temp_df = temp_df[["时间", "股票代码", "相关股票代码", "涨跌幅"]]
    temp_df["涨跌幅"] = temp_df["涨跌幅"].str.strip("%")
    temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    stock_hot_rank_em_df = stock_hot_rank_em()
    print(stock_hot_rank_em_df)

    stock_hot_rank_detail_em_df = stock_hot_rank_detail_em(symbol="SZ871245")
    print(stock_hot_rank_detail_em_df)

    stock_hot_rank_detail_realtime_em_df = stock_hot_rank_detail_realtime_em(
        symbol="SZ000665"
    )
    print(stock_hot_rank_detail_realtime_em_df)

    stock_hot_keyword_em_df = stock_hot_keyword_em(symbol="SZ000665")
    print(stock_hot_keyword_em_df)

    stock_hot_rank_latest_em_df = stock_hot_rank_latest_em(symbol="SZ000665")
    print(stock_hot_rank_latest_em_df)

    stock_hot_rank_relate_em_df = stock_hot_rank_relate_em(symbol="SZ000665")
    print(stock_hot_rank_relate_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2022/6/28 14:57
Desc: 中国外汇交易中心暨全国银行间同业拆借中心-市场数据-市场行情-外汇市场行情
人民币外汇即期报价: fx_spot_quote
人民币外汇远掉报价: fx_swap_quote
外币对即期报价: fx_pair_quote
"""

import time

import pandas as pd
import requests

from akshare.fx.cons import (
    SHORT_HEADERS,
    FX_SPOT_URL,
    FX_SWAP_URL,
    FX_PAIR_URL,
)


def fx_spot_quote() -> pd.DataFrame:
    """
    中国外汇交易中心暨全国银行间同业拆借中心-市场数据-市场行情-外汇市场行情-人民币外汇即期报价
    http://www.chinamoney.com.cn/chinese/mkdatapfx/
    :return: 人民币外汇即期报价
    :rtype: pandas.DataFrame
    """
    payload = {"t": str(int(round(time.time() * 1000)))}
    res = requests.post(FX_SPOT_URL, data=payload, headers=SHORT_HEADERS)
    temp_df = pd.DataFrame(res.json()["records"])
    temp_df = temp_df[["ccyPair", "bidPrc", "askPrc", "midprice", "time"]]
    temp_df.columns = [
        "货币对",
        "买报价",
        "卖报价",
        "-",
        "-",
    ]
    temp_df = temp_df[["货币对", "买报价", "卖报价"]]
    temp_df["买报价"] = pd.to_numeric(temp_df["买报价"], errors="coerce")
    temp_df["卖报价"] = pd.to_numeric(temp_df["卖报价"], errors="coerce")
    return temp_df


def fx_swap_quote() -> pd.DataFrame:
    """
    中国外汇交易中心暨全国银行间同业拆借中心-市场数据-市场行情-债券市场行情-人民币外汇远掉报价
    https://www.chinamoney.com.cn/chinese/index.html
    :return: 人民币外汇远掉报价
    :rtype: pandas.DataFrame
    """
    payload = {"t": str(int(round(time.time() * 1000)))}
    res = requests.post(FX_SWAP_URL, data=payload, headers=SHORT_HEADERS)
    temp_df = pd.DataFrame(res.json()["records"])
    temp_df = temp_df[
        [
            "ccyPair",
            "label_1W",
            "label_1M",
            "label_3M",
            "label_6M",
            "label_9M",
            "label_1Y",
        ]
    ]
    temp_df.columns = [
        "货币对",
        "1周",
        "1月",
        "3月",
        "6月",
        "9月",
        "1年",
    ]
    return temp_df


def fx_pair_quote() -> pd.DataFrame:
    """
    中国外汇交易中心暨全国银行间同业拆借中心-市场数据-市场行情-债券市场行情-外币对即期报价
    http://www.chinamoney.com.cn/chinese/mkdatapfx/
    :return: 外币对即期报价
    :rtype: pandas.DataFrame
    """
    payload = {"t": str(int(round(time.time() * 1000)))}
    res = requests.post(FX_PAIR_URL, data=payload, headers=SHORT_HEADERS)
    temp_df = pd.DataFrame(res.json()["records"])
    temp_df = temp_df[["ccyPair", "bidPrc", "askPrc", "midprice", "time"]]
    temp_df.columns = [
        "货币对",
        "买报价",
        "卖报价",
        "-",
        "-",
    ]
    temp_df = temp_df[["货币对", "买报价", "卖报价"]]
    temp_df["买报价"] = pd.to_numeric(temp_df["买报价"], errors="coerce")
    temp_df["卖报价"] = pd.to_numeric(temp_df["卖报价"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    fx_spot_quote_df = fx_spot_quote()
    print(fx_spot_quote_df)

    fx_swap_quote_df = fx_swap_quote()
    print(fx_swap_quote_df)

    fx_pair_quote_df = fx_pair_quote()
    print(fx_pair_quote_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/3/4 20:00
Desc: 同花顺-财务指标-主要指标
https://basic.10jqka.com.cn/new/000063/finance.html
"""

import json

import pandas as pd
import requests
from bs4 import BeautifulSoup

from akshare.utils.cons import headers


def stock_financial_abstract_ths(
    symbol: str = "000063", indicator: str = "按报告期"
) -> pd.DataFrame:
    """
    同花顺-财务指标-主要指标
    https://basic.10jqka.com.cn/new/000063/finance.html
    :param symbol: 股票代码
    :type symbol: str
    :param indicator: 指标; choice of {"按报告期", "按年度", "按单季度"}
    :type indicator: str
    :return: 同花顺-财务指标-主要指标
    :rtype: pandas.DataFrame
    """
    url = f"https://basic.10jqka.com.cn/new/{symbol}/finance.html"
    r = requests.get(url, headers=headers)
    soup = BeautifulSoup(r.text, features="lxml")
    data_text = soup.find(name="p", attrs={"id": "main"}).string
    data_json = json.loads(data_text)
    df_index = [
        item[0] if isinstance(item, list) else item for item in data_json["title"]
    ]
    if indicator == "按报告期":
        temp_df = pd.DataFrame(
            data_json["report"][1:], columns=data_json["report"][0], index=df_index[1:]
        )
    elif indicator == "按单季度":
        temp_df = pd.DataFrame(
            data_json["simple"][1:], columns=data_json["simple"][0], index=df_index[1:]
        )
    else:
        temp_df = pd.DataFrame(
            data_json["year"][1:], columns=data_json["year"][0], index=df_index[1:]
        )
    temp_df = temp_df.T
    temp_df.reset_index(inplace=True)
    temp_df.rename(columns={"index": "报告期"}, inplace=True)
    temp_df.sort_values(by="报告期", ignore_index=True, inplace=True)
    return temp_df


def stock_financial_debt_ths(
    symbol: str = "000063", indicator: str = "按报告期"
) -> pd.DataFrame:
    """
    同花顺-财务指标-资产负债表
    https://basic.10jqka.com.cn/new/000063/finance.html
    https://basic.10jqka.com.cn/api/stock/finance/000063_debt.json
    :param symbol: 股票代码
    :type symbol: str
    :param indicator: 指标; choice of {"按报告期", "按年度"}
    :type indicator: str
    :return: 同花顺-财务指标-资产负债表
    :rtype: pandas.DataFrame
    """
    url = f"https://basic.10jqka.com.cn/api/stock/finance/{symbol}_debt.json"
    r = requests.get(url, headers=headers)
    data_json = json.loads(json.loads(r.text)["flashData"])
    df_index = [
        item[0] if isinstance(item, list) else item for item in data_json["title"]
    ]
    if indicator == "按报告期":
        temp_df = pd.DataFrame(
            data_json["report"][1:], columns=data_json["report"][0], index=df_index[1:]
        )
    else:
        temp_df = pd.DataFrame(
            data_json["year"][1:], columns=data_json["year"][0], index=df_index[1:]
        )
    temp_df = temp_df.T
    temp_df.reset_index(inplace=True)
    temp_df.rename(columns={"index": "报告期"}, inplace=True)
    return temp_df


def stock_financial_benefit_ths(
    symbol: str = "000063", indicator: str = "按报告期"
) -> pd.DataFrame:
    """
    同花顺-财务指标-利润表
    https://basic.10jqka.com.cn/new/000063/finance.html
    https://basic.10jqka.com.cn/api/stock/finance/000063_benefit.json
    :param symbol: 股票代码
    :type symbol: str
    :param indicator: 指标; choice of {"按报告期","按单季度", "按年度"}
    :type indicator: str
    :return: 同花顺-财务指标-利润表
    :rtype: pandas.DataFrame
    """
    url = f"https://basic.10jqka.com.cn/api/stock/finance/{symbol}_benefit.json"
    r = requests.get(url, headers=headers)
    data_json = json.loads(json.loads(r.text)["flashData"])
    df_index = [
        item[0] if isinstance(item, list) else item for item in data_json["title"]
    ]
    if indicator == "按报告期":
        temp_df = pd.DataFrame(
            data_json["report"][1:], columns=data_json["report"][0], index=df_index[1:]
        )
    elif indicator == "按单季度":
        temp_df = pd.DataFrame(
            data_json["simple"][1:], columns=data_json["simple"][0], index=df_index[1:]
        )
    else:
        temp_df = pd.DataFrame(
            data_json["year"][1:], columns=data_json["year"][0], index=df_index[1:]
        )
    temp_df = temp_df.T
    temp_df.reset_index(inplace=True)
    temp_df.rename(columns={"index": "报告期"}, inplace=True)
    return temp_df


def stock_financial_cash_ths(
    symbol: str = "000063", indicator: str = "按报告期"
) -> pd.DataFrame:
    """
    同花顺-财务指标-现金流量表
    https://basic.10jqka.com.cn/new/000063/finance.html
    https://basic.10jqka.com.cn/api/stock/finance/000063_cash.json
    :param symbol: 股票代码
    :type symbol: str
    :param indicator: 指标; choice of {"按报告期","按单季度", "按年度"}
    :type indicator: str
    :return: 同花顺-财务指标-现金流量表
    :rtype: pandas.DataFrame
    """
    url = f"https://basic.10jqka.com.cn/api/stock/finance/{symbol}_cash.json"
    r = requests.get(url, headers=headers)
    data_json = json.loads(json.loads(r.text)["flashData"])
    df_index = [
        item[0] if isinstance(item, list) else item for item in data_json["title"]
    ]
    if indicator == "按报告期":
        temp_df = pd.DataFrame(
            data_json["report"][1:], columns=data_json["report"][0], index=df_index[1:]
        )
    elif indicator == "按单季度":
        temp_df = pd.DataFrame(
            data_json["simple"][1:], columns=data_json["simple"][0], index=df_index[1:]
        )
    else:
        temp_df = pd.DataFrame(
            data_json["year"][1:], columns=data_json["year"][0], index=df_index[1:]
        )
    temp_df = temp_df.T
    temp_df.reset_index(inplace=True)
    temp_df.rename(columns={"index": "报告期"}, inplace=True)
    return temp_df


def stock_management_change_ths(symbol: str = "688981") -> pd.DataFrame:
    """
    同花顺-公司大事-高管持股变动
    https://basic.10jqka.com.cn/new/688981/event.html
    :param symbol: 股票代码
    :type symbol: str
    :return: 同花顺-公司大事-高管持股变动
    :rtype: pandas.DataFrame
    """
    url = f"https://basic.10jqka.com.cn/new/{symbol}/event.html"
    r = requests.get(url, headers=headers)
    r.encoding = "gb2312"
    soup = BeautifulSoup(r.text, features="lxml")
    soup_find = soup.find(name="table", attrs={"class": "data_table_1 m_table m_hl"})
    if soup_find is not None:
        content_list = [item.text.strip() for item in soup_find]
        column_names = content_list[1].split("\n")
        row = (
            content_list[3]
            .replace(" ", "")
            .replace("\t", "")
            .replace("\n\n", "")
            .replace("   ", "\n")
            .replace("\n\n", "\n")
            .split("\n")
        )
        row = [item for item in row if item != ""]
        new_rows = []
        step = len(column_names)
        for i in range(0, len(row), step):
            new_rows.append(row[i : i + step])
        temp_df = pd.DataFrame(new_rows, columns=column_names)
        temp_df.sort_values(by="变动日期", ignore_index=True, inplace=True)
        temp_df["变动日期"] = pd.to_datetime(
            temp_df["变动日期"], errors="coerce"
        ).dt.date
        temp_df.rename(
            columns={
                "变动数量（股）": "变动数量",
                "交易均价（元）": "交易均价",
                "剩余股数（股）": "剩余股数",
            },
            inplace=True,
        )
        return temp_df
    return pd.DataFrame()


def stock_shareholder_change_ths(symbol: str = "688981") -> pd.DataFrame:
    """
    同花顺-公司大事-股东持股变动
    https://basic.10jqka.com.cn/new/688981/event.html
    :param symbol: 股票代码
    :type symbol: str
    :return: 同花顺-公司大事-股东持股变动
    :rtype: pandas.DataFrame
    """
    url = f"https://basic.10jqka.com.cn/new/{symbol}/event.html"
    r = requests.get(url, headers=headers)
    r.encoding = "gb2312"
    soup = BeautifulSoup(r.text, features="lxml")
    soup_find = soup.find(name="table", attrs={"class": "m_table data_table_1 m_hl"})
    if soup_find is not None:
        content_list = [item.text.strip() for item in soup_find]
        column_names = content_list[1].split("\n")
        row = (
            content_list[3]
            .replace("\t", "")
            .replace("\n\n", "")
            .replace("   ", "\n")
            .replace(" ", "")
            .replace("\n\n", "\n")
            .split("\n")
        )
        row = [item for item in row if item != ""]
        new_rows = []
        step = len(column_names)
        for i in range(0, len(row), step):
            new_rows.append(row[i : i + step])
        temp_df = pd.DataFrame(new_rows, columns=column_names)
        temp_df.sort_values(by="公告日期", ignore_index=True, inplace=True)
        temp_df["公告日期"] = pd.to_datetime(
            temp_df["公告日期"], errors="coerce"
        ).dt.date
        temp_df.rename(
            columns={
                "变动数量(股)": "变动数量",
                "交易均价(元)": "交易均价",
                "剩余股份总数(股)": "剩余股份总数",
            },
            inplace=True,
        )
        return temp_df
    return pd.DataFrame()


if __name__ == "__main__":
    stock_financial_abstract_ths_df = stock_financial_abstract_ths(
        symbol="000063", indicator="按报告期"
    )
    print(stock_financial_abstract_ths_df)

    stock_financial_abstract_ths_df = stock_financial_abstract_ths(
        symbol="000063", indicator="按年度"
    )
    print(stock_financial_abstract_ths_df)

    stock_financial_abstract_ths_df = stock_financial_abstract_ths(
        symbol="000063", indicator="按单季度"
    )
    print(stock_financial_abstract_ths_df)

    stock_financial_debt_ths_df = stock_financial_debt_ths(
        symbol="002004", indicator="按报告期"
    )
    print(stock_financial_debt_ths_df)

    stock_financial_debt_ths_df = stock_financial_debt_ths(
        symbol="000063", indicator="按年度"
    )
    print(stock_financial_debt_ths_df)

    stock_financial_benefit_ths_df = stock_financial_benefit_ths(
        symbol="000063", indicator="按报告期"
    )
    print(stock_financial_benefit_ths_df)

    stock_financial_benefit_ths_df = stock_financial_benefit_ths(
        symbol="000063", indicator="按年度"
    )
    print(stock_financial_benefit_ths_df)

    stock_financial_benefit_ths_df = stock_financial_benefit_ths(
        symbol="000063", indicator="按报告期"
    )
    print(stock_financial_benefit_ths_df)

    stock_financial_cash_ths_df = stock_financial_cash_ths(
        symbol="000063", indicator="按年度"
    )
    print(stock_financial_cash_ths_df)

    stock_financial_cash_ths_df = stock_financial_cash_ths(
        symbol="000063", indicator="按年度"
    )
    print(stock_financial_cash_ths_df)

    stock_financial_cash_ths_df = stock_financial_cash_ths(
        symbol="000063", indicator="按单季度"
    )
    print(stock_financial_cash_ths_df)

    stock_management_change_ths_df = stock_management_change_ths(symbol="688981")
    print(stock_management_change_ths_df)

    stock_shareholder_change_ths_df = stock_shareholder_change_ths(symbol="688981")
    print(stock_shareholder_change_ths_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/7/25 14:30
Desc: 巨潮资讯-数据中心-新股数据
https://webapi.cninfo.com.cn/#/xinguList
"""

import pandas as pd
import py_mini_racer
import requests

from akshare.datasets import get_ths_js


def _get_file_content_cninfo(file: str = "cninfo.js") -> str:
    """
    获取 JS 文件的内容
    :param file:  JS 文件名
    :type file: str
    :return: 文件内容
    :rtype: str
    """
    setting_file_path = get_ths_js(file)
    with open(setting_file_path, encoding="utf-8") as f:
        file_data = f.read()
    return file_data


def stock_new_gh_cninfo() -> pd.DataFrame:
    """
    巨潮资讯-数据中心-新股数据-新股过会
    https://webapi.cninfo.com.cn/#/xinguList
    :return: 新股过会
    :rtype: pandas.DataFrame
    """
    url = "https://webapi.cninfo.com.cn/api/sysapi/p_sysapi1098"
    js_code = py_mini_racer.MiniRacer()
    js_content = _get_file_content_cninfo("cninfo.js")
    js_code.eval(js_content)
    mcode = js_code.call("getResCode1")
    headers = {
        "Accept": "*/*",
        "Accept-Enckey": mcode,
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Content-Length": "0",
        "Host": "webapi.cninfo.com.cn",
        "Origin": "https://webapi.cninfo.com.cn",
        "Pragma": "no-cache",
        "Proxy-Connection": "keep-alive",
        "Referer": "https://webapi.cninfo.com.cn/",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/93.0.4577.63 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
    }
    r = requests.post(url, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["records"])
    temp_df.columns = [
        "公司名称",
        "上会日期",
        "审核类型",
        "审议内容",
        "审核结果",
        "审核公告日",
    ]
    temp_df["上会日期"] = pd.to_datetime(temp_df["上会日期"], errors="coerce").dt.date
    temp_df["审核公告日"] = pd.to_datetime(
        temp_df["审核公告日"], errors="coerce"
    ).dt.date
    return temp_df


def stock_new_ipo_cninfo() -> pd.DataFrame:
    """
    巨潮资讯-数据中心-新股数据-新股发行
    https://webapi.cninfo.com.cn/#/xinguList
    :return: 新股发行
    :rtype: pandas.DataFrame
    """
    url = "https://webapi.cninfo.com.cn/api/sysapi/p_sysapi1097"
    js_code = py_mini_racer.MiniRacer()
    js_content = _get_file_content_cninfo("cninfo.js")
    js_code.eval(js_content)
    mcode = js_code.call("getResCode1")
    headers = {
        "Accept": "*/*",
        "Accept-Enckey": mcode,
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Content-Length": "0",
        "Host": "webapi.cninfo.com.cn",
        "Origin": "https://webapi.cninfo.com.cn",
        "Pragma": "no-cache",
        "Proxy-Connection": "keep-alive",
        "Referer": "https://webapi.cninfo.com.cn/",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/93.0.4577.63 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
    }
    params = {
        "timetype": "36",
        "market": "ALL",
    }
    r = requests.post(url, headers=headers, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["records"])
    temp_df.columns = [
        "摇号结果公告日",
        "中签公告日",
        "证券简称",
        "上市日期",
        "中签缴款日",
        "申购日期",
        "发行价",
        "证劵代码",
        "上网发行中签率",
        "总发行数量",
        "发行市盈率",
        "上网发行数量",
        "网上申购上限",
    ]
    temp_df = temp_df[
        [
            "证劵代码",
            "证券简称",
            "上市日期",
            "申购日期",
            "发行价",
            "总发行数量",
            "发行市盈率",
            "上网发行中签率",
            "摇号结果公告日",
            "中签公告日",
            "中签缴款日",
            "网上申购上限",
            "上网发行数量",
        ]
    ]
    temp_df["摇号结果公告日"] = pd.to_datetime(
        temp_df["摇号结果公告日"], errors="coerce"
    ).dt.date
    temp_df["中签公告日"] = pd.to_datetime(
        temp_df["中签公告日"], errors="coerce"
    ).dt.date
    temp_df["上市日期"] = pd.to_datetime(temp_df["上市日期"], errors="coerce").dt.date
    temp_df["中签缴款日"] = pd.to_datetime(
        temp_df["中签缴款日"], errors="coerce"
    ).dt.date
    temp_df["申购日期"] = pd.to_datetime(temp_df["申购日期"], errors="coerce").dt.date
    temp_df["发行价"] = pd.to_numeric(temp_df["发行价"], errors="coerce")
    temp_df["上网发行中签率"] = pd.to_numeric(
        temp_df["上网发行中签率"], errors="coerce"
    )
    temp_df["总发行数量"] = pd.to_numeric(temp_df["总发行数量"], errors="coerce")
    temp_df["发行市盈率"] = pd.to_numeric(temp_df["发行市盈率"], errors="coerce")
    temp_df["上网发行数量"] = pd.to_numeric(temp_df["上网发行数量"], errors="coerce")
    temp_df["网上申购上限"] = pd.to_numeric(temp_df["网上申购上限"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    stock_new_gh_cninfo_df = stock_new_gh_cninfo()
    print(stock_new_gh_cninfo_df)

    stock_new_ipo_cninfo_df = stock_new_ipo_cninfo()
    print(stock_new_ipo_cninfo_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/3/22 18:00
Desc: 东方财富网-行情中心-沪深京板块-概念板块-名称
https://quote.eastmoney.com/center/boardlist.html#concept_board
异步接口-测试版
"""

import asyncio
from typing import Dict, List

import aiohttp
import pandas as pd


async def fetch_single_page(
    session: aiohttp.ClientSession, url: str, params: Dict
) -> Dict:
    """异步获取单页数据"""
    async with session.get(url, params=params, ssl=False) as response:
        return await response.json()


async def fetch_all_pages_async(url: str, base_params: Dict) -> List[Dict]:
    """异步获取所有页面数据"""
    # 首先获取总数以计算页数
    first_page_params = base_params.copy()
    first_page_params["pn"] = "1"

    async with aiohttp.ClientSession() as session:
        first_page_data = await fetch_single_page(session, url, first_page_params)

        # 检查是否成功获取数据
        if first_page_data.get("rc") != 0 or not first_page_data.get("data"):
            return [first_page_data]  # 返回错误信息

        total = first_page_data["data"]["total"]
        page_size = int(base_params["pz"])
        total_pages = (total + page_size - 1) // page_size

        # 限制页数，避免过大请求
        total_pages = min(total_pages, 100)

        # 创建所有页面的任务
        tasks = []
        for page in range(1, total_pages + 1):
            page_params = base_params.copy()
            page_params["pn"] = str(page)
            tasks.append(fetch_single_page(session, url, page_params))

        # 并发执行所有任务
        results = await asyncio.gather(*tasks)
        return results


def process_concept_board_data(page_results: List[Dict]) -> pd.DataFrame:
    """处理概念板块数据，转换为DataFrame"""
    all_data = []

    for result in page_results:
        if result.get("data") and result["data"].get("diff"):
            page_data = result["data"]["diff"]
            all_data.extend(page_data)

    if not all_data:
        return pd.DataFrame()

    temp_df = pd.DataFrame(all_data)

    # 转换数值类型，确保排序正确
    numeric_columns = ["f2", "f3", "f4", "f8", "f20", "f104", "f105", "f136"]
    for col in numeric_columns:
        if col in temp_df.columns:
            temp_df[col] = pd.to_numeric(temp_df[col], errors="coerce")

    # 按涨跌幅(f3)降序排序
    if "f3" in temp_df.columns:
        temp_df.sort_values(by="f3", ascending=False, inplace=True)

    # 重命名列
    columns_map = {
        "f2": "最新价",
        "f3": "涨跌幅",
        "f4": "涨跌额",
        "f8": "换手率",
        "f12": "板块代码",
        "f14": "板块名称",
        "f20": "总市值",
        "f104": "上涨家数",
        "f105": "下跌家数",
        "f128": "领涨股票",
        "f136": "领涨股票-涨跌幅",
    }

    # 选择需要的列并重命名
    selected_columns = list(columns_map.keys())
    available_columns = [col for col in selected_columns if col in temp_df.columns]
    temp_df = temp_df[available_columns]
    temp_df.rename(columns=columns_map, inplace=True)

    # 重置索引并添加排名列
    temp_df.reset_index(drop=True, inplace=True)
    temp_df.insert(0, "排名", range(1, len(temp_df) + 1))

    # 调整列顺序，与原函数保持一致
    final_columns = [
        "排名",
        "板块名称",
        "板块代码",
        "最新价",
        "涨跌额",
        "涨跌幅",
        "总市值",
        "换手率",
        "上涨家数",
        "下跌家数",
        "领涨股票",
        "领涨股票-涨跌幅",
    ]

    # 只保留存在的列
    final_columns = [col for col in final_columns if col in temp_df.columns]
    temp_df = temp_df[final_columns]

    return temp_df


async def stock_board_concept_name_em_async() -> pd.DataFrame:
    """
    异步获取东方财富网-行情中心-沪深京板块-概念板块-名称
    https://quote.eastmoney.com/center/boardlist.html#concept_board
    :return: 概念板块-名称
    :rtype: pandas.DataFrame
    """
    url = "https://79.push2.eastmoney.com/api/qt/clist/get"
    params = {
        "pn": "1",
        "pz": "100",
        "po": "1",  # 按涨跌幅排序，1为降序
        "np": "1",
        "ut": "bd1d9ddb04089700cf9c27f6f7426281",
        "fltt": "2",
        "invt": "2",
        "fid": "f3",  # 按涨跌幅排序
        "fs": "m:90 t:3 f:!50",
        "fields": "f2,f3,f4,f8,f12,f14,f15,f16,f17,f18,f20,f21,f24,f25,f22,f33,f11,f62,f128,f124,f107,f104,f105,f136",
    }

    results = await fetch_all_pages_async(url, params)
    return process_concept_board_data(results)


def stock_board_concept_name_em() -> pd.DataFrame:
    """
    东方财富网-行情中心-沪深京板块-概念板块-名称 (同步接口)
    https://quote.eastmoney.com/center/boardlist.html#concept_board
    :return: 概念板块-名称
    :rtype: pandas.DataFrame
    """
    import nest_asyncio

    nest_asyncio.apply()
    return asyncio.run(stock_board_concept_name_em_async())


if __name__ == "__main__":
    # 测试同步接口
    stock_board_concept_name_em_df = stock_board_concept_name_em()
    print(stock_board_concept_name_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2023/8/14 11:10
Desc: 国家金融与发展实验室-中国宏观杠杆率数据
http://114.115.232.154:8080/
"""

import pandas as pd


def macro_cnbs() -> pd.DataFrame:
    """
    国家金融与发展实验室-中国宏观杠杆率数据
    http://114.115.232.154:8080/
    :return: 中国宏观杠杆率数据
    :rtype: pandas.DataFrame
    """
    url = "http://114.115.232.154:8080/handler/download.ashx"
    temp_df = pd.read_excel(
        url, sheet_name="Data", header=0, skiprows=1, engine="openpyxl"
    )

    temp_df["Period"] = pd.to_datetime(temp_df["Period"]).dt.strftime("%Y-%m")
    temp_df.dropna(axis=1, inplace=True)

    temp_df.rename(
        columns={
            "Period": "年份",
            "Household": "居民部门",
            "Non-financial corporations": "非金融企业部门",
            "Central government ": "中央政府",
            "Local government": "地方政府",
            "General government": "政府部门",
            "Non financial sector": "实体经济部门",
            "Financial sector(asset side)": "金融部门资产方",
            "Financial sector(liability side)": "金融部门负债方",
        },
        inplace=True,
    )

    column_order = [
        "年份",
        "居民部门",
        "非金融企业部门",
        "政府部门",
        "中央政府",
        "地方政府",
        "实体经济部门",
        "金融部门资产方",
        "金融部门负债方",
    ]
    temp_df = temp_df.reindex(columns=column_order)
    temp_df["居民部门"] = pd.to_numeric(temp_df["居民部门"], errors="coerce")
    temp_df["非金融企业部门"] = pd.to_numeric(
        temp_df["非金融企业部门"], errors="coerce"
    )
    temp_df["政府部门"] = pd.to_numeric(temp_df["政府部门"], errors="coerce")
    temp_df["中央政府"] = pd.to_numeric(temp_df["中央政府"], errors="coerce")
    temp_df["地方政府"] = pd.to_numeric(temp_df["地方政府"], errors="coerce")
    temp_df["实体经济部门"] = pd.to_numeric(temp_df["实体经济部门"], errors="coerce")
    temp_df["金融部门资产方"] = pd.to_numeric(
        temp_df["金融部门资产方"], errors="coerce"
    )
    temp_df["金融部门负债方"] = pd.to_numeric(
        temp_df["金融部门负债方"], errors="coerce"
    )
    return temp_df


if __name__ == "__main__":
    macro_cnbs_df = macro_cnbs()
    print(macro_cnbs_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2023/6/19 18:16
Desc: 浙江省排污权交易指数
https://zs.zjpwq.net/
"""

import pandas as pd
import requests


def index_eri(symbol: str = "月度") -> pd.DataFrame:
    """
    浙江省排污权交易指数
    https://zs.zjpwq.net
    :param symbol: choice of {"月度", "季度"}
    :type symbol: str
    :return: 浙江省排污权交易指数
    :rtype: pandas.DataFrame
    """
    symbol_map = {
        "月度": "MONTH",
        "季度": "QUARTER",
    }
    url = "https://zs.zjpwq.net/pwq-index-webapi/indexData"
    params = {
        "cycle": symbol_map[symbol],
        "regionId": "1",
        "structId": "1",
        "pageSize": "5000",
        "indexId": "1",
        "orderBy": "stage.publishTime",
    }
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/114.0.0.0 Safari/537.36"
    }
    r = requests.get(url, params=params, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"])
    index_value = temp_df["indexValue"].tolist()
    index_time = [item["stage"]["publishTime"] for item in data_json["data"]]
    big_df = pd.DataFrame([index_time, index_value], index=["日期", "交易指数"]).T
    url = "https://zs.zjpwq.net/pwq-index-webapi/dataStatistics"
    r = requests.get(url, params=params, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"])
    big_df["成交量"] = temp_df["totalQuantity"].tolist()
    big_df["成交额"] = temp_df["totalCost"].tolist()
    big_df["日期"] = pd.to_datetime(big_df["日期"], errors="coerce").dt.date
    big_df["交易指数"] = pd.to_numeric(big_df["交易指数"], errors="coerce")
    big_df["成交量"] = pd.to_numeric(big_df["成交量"], errors="coerce")
    big_df["成交额"] = pd.to_numeric(big_df["成交额"], errors="coerce")
    return big_df


if __name__ == "__main__":
    index_eri_df = index_eri(symbol="月度")
    print(index_eri_df)

    index_eri_df = index_eri(symbol="季度")
    print(index_eri_df)
